#include "cquad.h"

double cquad_dqagp(double f(), double a, double b, int npts2, double *points, double epsabs, double epsrel, double *abserr, int *neval, int *ier, int *last, int *iwork, double *work)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqagp     ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f over (a,b), hopefully satisfying          ***
***    following claim for accuracy:                                        ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***                                                                         ***
***    Break points of the integration interval, where local difficulties   ***
***    of the integrand may occur (e.g.singularities, discontinuities),     ***
***    are provided by the user.                                            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    An integrator based on globally adaptive interval subdivision in     ***
***    connection with extrapolation by the Epsilon algorithm. Allows for   ***
***    eventual user-supplied information, i.e. the abscissae of internal   ***
***    singularities, discontinuities and other difficulties of the         ***
***    integrand function.                                                  ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration                                  ***
***                                                                         ***
***    npts2    number of user-supplied break points within the integration ***
***             range increased by 2                                        ***
***             (npts2 > 1)                                                 ***
***                                                                         ***
***    points   vector of dimension npts2, the first (npts2 - 2) elements   ***
***             of which are the user provided break points. If these       ***
***             points do not constitute an ascending sequence there will   ***
***             be an automatic sorting.                                    ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                           The error may be under-estimated.             ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 4 --> the algorithm does not converge. Roundoff     ***
***                           error is detected in the extrapolation table. ***
***                           It is presumed that the requested tolerance   ***
***                           cannot be achieved, and that the returned     ***
***                           result is the best which can be obtained.     ***
***                                                                         ***
***               ier = 5 --> the integral is probably divergent, or slowly ***
***                           convergent. it must be noted that divergence  ***
***                           can occur with any other value of ier > 0.    ***
***                                                                         ***
***               ier = 6 --> the input is invalid because npts2 < 2 or     ***
***                           break points are specified outside the        ***
***                           the integration range or epsabs <= 0 and      ***
***                           epsrel < max (50 * rel.mach.acc., 0.5d-28).   ***
***                           In all these events res, abserr, neval and    ***
***                           last are set to zero.                         ***
***                                                                         ***
***    last     number of subintervals produced in the subdivision process  ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    leniw    dimensioning value for iwork.                               ***
***             leniw determines limit = (leniw - npts2) / 2, which is the  ***
***             maximum number of subintervals in the partition of the      ***
***             given integration interval (a,b).                           ***
***             (leniw >= (3 * npts2 - 2))                                  ***
***                                                                         ***
***    lenw     dimensioning value for work.                                ***
***             (lenw must be at least leniw * 2 - npts2)                   ***
***                                                                         ***
***    iwork    vector of dimension at least leniw.                         ***
***             On return:                                                  ***
***                the first k elements of iwork contain pointers to the    ***
***                  error estimates over the subintervals, such that       ***
***                  work(limit * 3 + iwork(1)), ..., work(limit * 3 +      ***
***                  iwork(k)) form a decreasing sequence, with k = last    ***
***                  if last <= (limit/2 + 2), and k = limit + 1 - last     ***
***                  otherwise                                              ***
***                iwork(limit + 1), ..., iwork(limit + last) contain the   ***
***                  subdivision levels of the subintervals, i.e. if        ***
***                  (aa, bb) is a subinterval of (p1, p2) where p1 as well ***
***                  as p2 is a user-provided break point or integration    ***
***                  limit, then (aa, bb) has level l if                    ***
***                  abs(bb - aa) = abs(p2 - p1) * 2 * *(-l)                *** 
***                iwork(limit*2+1), ..., iwork(limit*2+npts2) have no      ***
***                  significance for the user                              ***
***                (note that limit = (leniw-npts2) / 2)                    ***
***                                                                         ***
***    work     vector of dimension at least lenw.                          ***
***             On return:                                                  ***
***                work(1), ..., work(last) contain the left end points of  ***
***                  the subintervals in the partition of (a,b)             ***
***                work(limit + 1), ..., work(limit + last) contain the     ***
***                  right end points                                       ***
***                work(limit * 2 + 1), ..., work(limit * 2 + last) contain ***
***                  the integral approximations over the subintervals      ***
***                work(limit * 3 + 1), ..., work(limit * 3 + last) contain ***
***                  the the corresponding error estimates                  ***
***                work(limit * 4 + 1), ..., work(limit * 4 + npts2)        ***
***                  contain the integration limits and the break points    ***
***                  sorted in an ascending sequence                        ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int lenw = LENW, leniw = LENIW;
    double res;

    dqagp_(f, &a, &b, &npts2, points, &epsabs, &epsrel, &res, abserr, neval, ier, &leniw, &lenw, last, iwork, work);

    return res;
}
